@@ -49,6 +49,7 @@ from optax._src.clipping import clip_by_block_rms
 from optax._src.clipping import clip_by_global_norm
 from optax._src.clipping import ClipByGlobalNormState
 from optax._src.clipping import ClipState
+from optax._src.clipping import per_example_global_norm_clip
 from optax._src.combine import chain
 from optax._src.combine import multi_transform
 from optax._src.combine import MultiTransformState
@@ -77,6 +78,7 @@ from optax._src.loss import log_cosh
 from optax._src.loss import sigmoid_binary_cross_entropy
 from optax._src.loss import smooth_labels
 from optax._src.loss import softmax_cross_entropy
+from optax._src.loss import softmax_cross_entropy_with_integer_labels
 from optax._src.numerics import safe_int32_increment
 from optax._src.numerics import safe_norm
 from optax._src.numerics import safe_root_mean_squares
@@ -111,6 +113,7 @@ from optax._src.transform import AdditiveWeightDecayState
 from optax._src.transform import AddNoiseState
 from optax._src.transform import apply_every
 from optax._src.transform import ApplyEvery
+from optax._src.transform import bias_correction
 from optax._src.transform import centralize
 from optax._src.transform import ema
 from optax._src.transform import EmaState
@@ -139,6 +142,9 @@ from optax._src.transform import ScaleByTrustRatioState
 from optax._src.transform import ScaleState
 from optax._src.transform import trace
 from optax._src.transform import TraceState
+from optax._src.transform import update_infinity_moment
+from optax._src.transform import update_moment
+from optax._src.transform import update_moment_per_elem_norm
 from optax._src.update import apply_updates
 from optax._src.update import incremental_update
 from optax._src.update import periodic_update
@@ -148,13 +154,17 @@ from optax._src.wrappers import apply_if_finite
 from optax._src.wrappers import ApplyIfFiniteState
 from optax._src.wrappers import flatten
 from optax._src.wrappers import masked
+from optax._src.wrappers import MaskedNode
 from optax._src.wrappers import MaskedState
 from optax._src.wrappers import maybe_update
 from optax._src.wrappers import MaybeUpdateState
 from optax._src.wrappers import MultiSteps
 from optax._src.wrappers import MultiStepsState
+from optax._src.wrappers import ShouldSkipUpdateFunction
+from optax._src.wrappers import skip_large_updates
+from optax._src.wrappers import skip_not_finite
 
-__version__ = "0.1.2"
+__version__ = "0.1.3"
 
 __all__ = (
     "adabelief",
@@ -240,6 +250,7 @@ __all__ = (
     "Params",
     "pathwise_jacobians",
     "periodic_update",
+    "per_example_global_norm_clip",
     "piecewise_constant_schedule",
     "piecewise_interpolate_schedule",
     "polynomial_schedule",
@@ -279,7 +290,10 @@ __all__ = (
     "set_to_zero",
     "sgd",
     "sgdr_schedule",
+    "ShouldSkipUpdateFunction",
     "sigmoid_binary_cross_entropy",
+    "skip_large_updates",
+    "skip_not_finite",
     "sm3",
     "smooth_labels",
     "softmax_cross_entropy",
