@@ -69,6 +69,8 @@ class AliasTest(chex.TestCase):
           dict(opt_name='adagrad', opt=lambda: alias.adagrad(1.0)),
           dict(opt_name='adam', opt=lambda: alias.adam(1e-1)),
           dict(opt_name='adamw', opt=lambda: alias.adamw(1e-1)),
+          dict(opt_name='adamax', opt=lambda: alias.adam(1e-1)),
+          dict(opt_name='adamaxw', opt=lambda: alias.adamw(1e-1)),
           dict(opt_name='lars', opt=lambda: alias.lars(1.0)),
           dict(opt_name='lamb', opt=lambda: alias.lamb(1e-3)),
           dict(
@@ -106,7 +108,7 @@ class AliasTest(chex.TestCase):
         updates = updates[None]
       # Complex gradients need to be conjugated before being added to parameters
       # https://gist.github.com/wdphy16/118aef6fb5f82c49790d7678cf87da29
-      updates = jax.tree_map(lambda x: x.conj(), updates)
+      updates = jax.tree_util.tree_map(lambda x: x.conj(), updates)
       updates, state = opt.update(updates, state, params)
       params = update.apply_updates(params, updates)
       return params, state
