@@ -1,4 +1,3 @@
-# Lint as: python3
 # Copyright 2019 DeepMind Technologies Limited. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -47,7 +46,7 @@ def _test_optimizer(step_size: float) -> base.GradientTransformation:
   # Use SGD for simplicity but add non-trivial optimizer state so that the
   # resetting behaviour of lookahead can be tested.
   def init_fn(params):
-    aggregate_grads = jax.tree_map(jnp.zeros_like, params)
+    aggregate_grads = jax.tree_util.tree_map(jnp.zeros_like, params)
     return TestOptimizerState(aggregate_grads, is_reset=True)
 
   def update_fn(updates, state, params):
@@ -55,7 +54,7 @@ def _test_optimizer(step_size: float) -> base.GradientTransformation:
     # have been passed correctly.
     chex.assert_trees_all_equal_shapes(updates, params)
     aggregate_grads = update.apply_updates(state.aggregate_grads, updates)
-    updates = jax.tree_map(lambda u: step_size * u, updates)
+    updates = jax.tree_util.tree_map(lambda u: step_size * u, updates)
     return updates, TestOptimizerState(aggregate_grads, is_reset=False)
 
   return base.GradientTransformation(init_fn, update_fn)
