@@ -50,6 +50,9 @@ def l2_loss(
     elementwise squared differences, with same shape as `predictions`.
   """
   chex.assert_type([predictions], float)
+  if targets is not None:
+    # Avoid broadcasting logic for "-" operator.
+    chex.assert_equal_shape((predictions, targets))
   errors = (predictions - targets) if (targets is not None) else predictions
   return 0.5 * (errors)**2
 
@@ -167,6 +170,41 @@ def softmax_cross_entropy(
   return -jnp.sum(labels * jax.nn.log_softmax(logits, axis=-1), axis=-1)
 
 
+def softmax_cross_entropy_with_integer_labels(
+    logits: chex.Array,
+    labels: chex.Array,
+) -> chex.Array:
+  """Computes softmax cross entropy between sets of logits and integer labels.
+
+  Measures the probability error in discrete classification tasks in which
+  the classes are mutually exclusive (each entry is in exactly one class).
+  For example, each CIFAR-10 image is labeled with one and only one label:
+  an image can be a dog or a truck, but not both.
+
+  References:
+    [Goodfellow et al, 2016](http://www.deeplearningbook.org/contents/prob.html)
+
+  Args:
+    logits: Unnormalized log probabilities, with shape `[..., num_classes]`.
+    labels: Integers specifying the correct class for each input, with shape
+      `[...]`.
+
+  Returns:
+    Cross entropy between each prediction and the corresponding target
+    distributions, with shape `[...]`.
+  """
+  chex.assert_type([logits], float)
+  chex.assert_type([labels], int)
+  # This is like jnp.take_along_axis(jax.nn.log_softmax(...), ...) except that
+  # we avoid subtracting the normalizer from all values, just from the values
+  # for the correct labels.
+  logits_max = jnp.max(logits, axis=-1, keepdims=True)
+  logits -= jax.lax.stop_gradient(logits_max)
+  label_logits = jnp.take_along_axis(logits, labels[..., None], axis=-1)[..., 0]
+  log_normalizers = jnp.log(jnp.sum(jnp.exp(logits), axis=-1))
+  return log_normalizers - label_logits
+
+
 def cosine_similarity(
     predictions: chex.Array,
     targets: chex.Array,
@@ -419,3 +457,49 @@ def ctc_loss(logits: chex.Array,
       logits, logit_paddings, labels, label_paddings,
       blank_id=blank_id, log_epsilon=log_epsilon)
   return per_seq_loss
+
+
+def kl_divergence(log_predictions: chex.Array,
+                  targets: chex.Array) -> chex.Array:
+  """Computes the Kullback-Leibler divergence (relative entropy) loss.
+
+  Measures the information gain achieved if target probability distribution
+  would be used instead of predicted probability distribution.
+
+  References:
+    [Kullback, Leibler, 1951](https://www.jstor.org/stable/2236703)
+
+  Args:
+    log_predictions: Probabilities of predicted distribution with shape
+      [..., dim]. Expected to be in the log-space to avoid underflow.
+    targets: Probabilities of target distribution with shape [..., dim].
+      Expected to be strictly positive.
+
+  Returns:
+    Kullback-Leibler divergence of predicted distribution from target
+    distribution with shape [...].
+  """
+  chex.assert_type([log_predictions, targets], float)
+  loss = targets * (jnp.log(targets) - log_predictions)
+  return jnp.sum(loss, axis=-1)
+
+
+def kl_divergence_with_log_targets(log_predictions: chex.Array,
+                                   log_targets: chex.Array) -> chex.Array:
+  """Computes the Kullback-Leibler divergence (relative entropy) loss.
+
+  Version of kl_div_loss where targets are given in log-space.
+
+  Args:
+    log_predictions: Probabilities of predicted distribution with shape
+      [..., dim]. Expected to be in the log-space to avoid underflow.
+    log_targets: Probabilities of target distribution with shape [..., dim].
+      Expected to be in the log-space.
+
+  Returns:
+    Kullback-Leibler divergence of predicted distribution from target
+    distribution with shape [...].
+  """
+  chex.assert_type([log_predictions, log_targets], float)
+  loss = jnp.exp(log_targets) * (log_targets - log_predictions)
+  return jnp.sum(loss, axis=-1)
