@@ -62,28 +62,35 @@ def trace(
 
   def init_fn(params):
     return TraceState(
-        trace=jax.tree_map(
+        trace=jax.tree_util.tree_map(
             lambda t: jnp.zeros_like(t, dtype=accumulator_dtype), params))
 
   def update_fn(updates, state, params=None):
     del params
     f = lambda g, t: g + decay * t
-    new_trace = jax.tree_map(f, updates, state.trace)
+    new_trace = jax.tree_util.tree_map(f, updates, state.trace)
     updates = (
-        jax.tree_map(f, updates, new_trace) if nesterov else new_trace)
+        jax.tree_util.tree_map(f, updates, new_trace) if nesterov
+        else new_trace)
     new_trace = utils.cast_tree(new_trace, accumulator_dtype)
     return updates, TraceState(trace=new_trace)
 
   return base.GradientTransformation(init_fn, update_fn)
 
 
-def _update_moment(updates, moments, decay, order):
+def update_moment(updates, moments, decay, order):
   """Compute the exponential moving average of the `order`-th moment."""
-  return jax.tree_map(
+  return jax.tree_util.tree_map(
       lambda g, t: (1 - decay) * (g ** order) + decay * t, updates, moments)
 
 
-def _update_moment_per_elem_norm(updates, moments, decay, order):
+def update_infinity_moment(updates, moments, decay, eps):
+  """Compute the exponential moving average of the infinity norm."""
+  return jax.tree_util.tree_map(
+      lambda g, t: jnp.maximum(jnp.abs(g) + eps, decay * t), updates, moments)
+
+
+def update_moment_per_elem_norm(updates, moments, decay, order):
   """Compute the EMA of the `order`-th moment of the element-wise norm."""
 
   def orderth_norm(g):
@@ -96,18 +103,19 @@ def _update_moment_per_elem_norm(updates, moments, decay, order):
         half_order = int(half_order)
       return _abs_sq(g) ** half_order
 
-  return jax.tree_map(
+  return jax.tree_util.tree_map(
       lambda g, t: (1 - decay) * orderth_norm(g) + decay * t, updates, moments)
 
 
-def _bias_correction(moment, decay, count):
+def bias_correction(moment, decay, count):
   """Perform bias correction. This becomes a no-op as count goes to infinity."""
-  bias_correction = 1 - decay**count
-  return jax.tree_map(lambda t: t / bias_correction.astype(t.dtype), moment)
+  bias_correction_ = 1 - decay**count
+  return jax.tree_util.tree_map(
+      lambda t: t / bias_correction_.astype(t.dtype), moment)
 
 
 def _reject_complex(params):
-  if any(jnp.iscomplexobj(x) for x in jax.tree_leaves(params)):
+  if any(jnp.iscomplexobj(x) for x in jax.tree_util.tree_leaves(params)):
     raise ValueError('This transformation does not support complex parameters.')
 
 
@@ -143,15 +151,15 @@ def ema(
   def init_fn(params):
     return EmaState(
         count=jnp.zeros([], jnp.int32),
-        ema=jax.tree_map(
+        ema=jax.tree_util.tree_map(
             lambda t: jnp.zeros_like(t, dtype=accumulator_dtype), params))
 
   def update_fn(updates, state, params=None):
     del params
-    updates = new_ema = _update_moment(updates, state.ema, decay, order=1)
+    updates = new_ema = update_moment(updates, state.ema, decay, order=1)
     count_inc = utils.safe_int32_increment(state.count)
     if debias:
-      updates = _bias_correction(new_ema, decay, count_inc)
+      updates = bias_correction(new_ema, decay, count_inc)
     state_ema = utils.cast_tree(new_ema, accumulator_dtype)
     return updates, EmaState(count=count_inc, ema=state_ema)
 
@@ -182,17 +190,17 @@ def scale_by_rss(
   """
 
   def init_fn(params):
-    sum_of_squares = jax.tree_map(
+    sum_of_squares = jax.tree_util.tree_map(
         lambda t: jnp.full_like(t, initial_accumulator_value), params)
     return ScaleByRssState(sum_of_squares=sum_of_squares)
 
   def update_fn(updates, state, params=None):
     del params
-    sum_of_squares = jax.tree_map(
+    sum_of_squares = jax.tree_util.tree_map(
         lambda g, t: _abs_sq(g) + t, updates, state.sum_of_squares)
-    inv_sqrt_g_square = jax.tree_map(
+    inv_sqrt_g_square = jax.tree_util.tree_map(
         lambda t: jnp.where(t > 0, jax.lax.rsqrt(t + eps), 0.0), sum_of_squares)
-    updates = jax.tree_map(
+    updates = jax.tree_util.tree_map(
         lambda scale, g: scale * g, inv_sqrt_g_square, updates)
     return updates, ScaleByRssState(sum_of_squares=sum_of_squares)
 
@@ -224,14 +232,14 @@ def scale_by_rms(
   """
 
   def init_fn(params):
-    nu = jax.tree_map(
+    nu = jax.tree_util.tree_map(
         lambda n: jnp.full_like(n, initial_scale), params)  # second moment
     return ScaleByRmsState(nu=nu)
 
   def update_fn(updates, state, params=None):
     del params
-    nu = _update_moment_per_elem_norm(updates, state.nu, decay, 2)
-    updates = jax.tree_map(
+    nu = update_moment_per_elem_norm(updates, state.nu, decay, 2)
+    updates = jax.tree_util.tree_map(
         lambda g, n: g * jax.lax.rsqrt(n + eps), updates, nu)
     return updates, ScaleByRmsState(nu=nu)
 
@@ -264,16 +272,16 @@ def scale_by_stddev(
   """
 
   def init_fn(params):
-    mu = jax.tree_map(jnp.zeros_like, params)  # First moment
-    nu = jax.tree_map(
+    mu = jax.tree_util.tree_map(jnp.zeros_like, params)  # First moment
+    nu = jax.tree_util.tree_map(
         lambda n: jnp.full_like(n, initial_scale), params)  # second moment
     return ScaleByRStdDevState(mu=mu, nu=nu)
 
   def update_fn(updates, state, params=None):
     del params
-    mu = _update_moment(updates, state.mu, decay, 1)
-    nu = _update_moment_per_elem_norm(updates, state.nu, decay, 2)
-    updates = jax.tree_map(
+    mu = update_moment(updates, state.mu, decay, 1)
+    nu = update_moment_per_elem_norm(updates, state.nu, decay, 2)
+    updates = jax.tree_util.tree_map(
         lambda g, m, n: g * jax.lax.rsqrt(n - _abs_sq(m) + eps),
         updates, mu, nu)
     return updates, ScaleByRStdDevState(mu=mu, nu=nu)
@@ -316,19 +324,19 @@ def scale_by_adam(
   mu_dtype = utils.canonicalize_dtype(mu_dtype)
 
   def init_fn(params):
-    mu = jax.tree_map(  # First moment
+    mu = jax.tree_util.tree_map(  # First moment
         lambda t: jnp.zeros_like(t, dtype=mu_dtype), params)
-    nu = jax.tree_map(jnp.zeros_like, params)  # Second moment
+    nu = jax.tree_util.tree_map(jnp.zeros_like, params)  # Second moment
     return ScaleByAdamState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
 
   def update_fn(updates, state, params=None):
     del params
-    mu = _update_moment(updates, state.mu, b1, 1)
-    nu = _update_moment_per_elem_norm(updates, state.nu, b2, 2)
+    mu = update_moment(updates, state.mu, b1, 1)
+    nu = update_moment_per_elem_norm(updates, state.nu, b2, 2)
     count_inc = numerics.safe_int32_increment(state.count)
-    mu_hat = _bias_correction(mu, b1, count_inc)
-    nu_hat = _bias_correction(nu, b2, count_inc)
-    updates = jax.tree_map(
+    mu_hat = bias_correction(mu, b1, count_inc)
+    nu_hat = bias_correction(nu, b2, count_inc)
+    updates = jax.tree_util.tree_map(
         lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
     mu = utils.cast_tree(mu, mu_dtype)
     return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
@@ -336,6 +344,43 @@ def scale_by_adam(
   return base.GradientTransformation(init_fn, update_fn)
 
 
+def scale_by_adamax(
+    b1: float = 0.9,
+    b2: float = 0.999,
+    eps: float = 1e-8
+) -> base.GradientTransformation:
+  """Rescale updates according to the Adamax algorithm.
+
+  References:
+    [Kingma et al, 2014](https://arxiv.org/abs/1412.6980)
+
+  Args:
+    b1: decay rate for the exponentially weighted average of grads.
+    b2: decay rate for the exponentially weighted maximum of grads.
+    eps: term added to the denominator to improve numerical stability.
+
+  Returns:
+    An (init_fn, update_fn) tuple.
+  """
+
+  def init_fn(params):
+    mu = jax.tree_util.tree_map(jnp.zeros_like, params)  # First moment
+    nu = jax.tree_util.tree_map(jnp.zeros_like, params)  # Infinite moment
+    return ScaleByAdamState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
+
+  def update_fn(updates, state, params=None):
+    del params
+    count_inc = numerics.safe_int32_increment(state.count)
+    mu = update_moment(updates, state.mu, b1, 1)
+    nu = update_infinity_moment(updates, state.nu, b2, eps)
+    # Bias correction for mean. No bias correction needed for infinity moment.
+    mu_hat = bias_correction(mu, b1, count_inc)
+    updates = jax.tree_util.tree_multimap(lambda m, v: m / v, mu_hat, nu)
+    return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
+
+  return base.GradientTransformation(init_fn, update_fn)
+
+
 ScaleState = base.EmptyState
 
 
@@ -357,7 +402,7 @@ def scale(
 
   def update_fn(updates, state, params=None):
     del params
-    updates = jax.tree_map(lambda g: step_size * g, updates)
+    updates = jax.tree_util.tree_map(lambda g: step_size * g, updates)
     return updates, state
 
   return base.GradientTransformation(init_fn, update_fn)
@@ -385,7 +430,7 @@ def scale_by_param_block_norm(
   def update_fn(updates, state, params):
     if params is None:
       raise ValueError(base.NO_PARAMS_MSG)
-    updates = jax.tree_map(
+    updates = jax.tree_util.tree_map(
         lambda u, p: u * numerics.safe_norm(p, min_scale),
         updates, params)
     return updates, state
@@ -415,7 +460,7 @@ def scale_by_param_block_rms(
   def update_fn(updates, state, params):
     if params is None:
       raise ValueError(base.NO_PARAMS_MSG)
-    updates = jax.tree_map(
+    updates = jax.tree_util.tree_map(
         lambda u, p: u * numerics.safe_root_mean_squares(p, min_scale),
         updates, params)
     return updates, state
@@ -454,20 +499,21 @@ def scale_by_belief(
   """
 
   def init_fn(params):
-    mu = jax.tree_map(jnp.zeros_like, params)  # First moment
-    s = jax.tree_map(jnp.zeros_like, params)  # Second Central moment
+    mu = jax.tree_util.tree_map(jnp.zeros_like, params)  # First moment
+    s = jax.tree_util.tree_map(jnp.zeros_like, params)  # Second Central moment
     return ScaleByBeliefState(count=jnp.zeros([], jnp.int32), mu=mu, nu=s)
 
   def update_fn(updates, state, params=None):
     del params
-    mu = _update_moment(updates, state.mu, b1, 1)
-    prediction_error = jax.tree_map(lambda g, m: g-m, updates, state.mu)
-    nu = _update_moment_per_elem_norm(prediction_error, state.nu, b2, 2)
-    nu = jax.tree_map(lambda v: v + eps_root, nu)
+    mu = update_moment(updates, state.mu, b1, 1)
+    prediction_error = jax.tree_util.tree_map(
+        lambda g, m: g-m, updates, state.mu)
+    nu = update_moment_per_elem_norm(prediction_error, state.nu, b2, 2)
+    nu = jax.tree_util.tree_map(lambda v: v + eps_root, nu)
     count_inc = numerics.safe_int32_increment(state.count)
-    mu_hat = _bias_correction(mu, b1, count_inc)
-    nu_hat = _bias_correction(nu, b2, count_inc)
-    updates = jax.tree_map(
+    mu_hat = bias_correction(mu, b1, count_inc)
+    nu_hat = bias_correction(nu, b2, count_inc)
+    updates = jax.tree_util.tree_map(
         lambda m, v: m / (jnp.sqrt(v) + eps), mu_hat, nu_hat)
     return updates, ScaleByBeliefState(count=count_inc, mu=mu, nu=nu)
 
@@ -504,20 +550,20 @@ def scale_by_yogi(
 
   def init_fn(params):
     value_like = lambda p: jnp.full_like(p, initial_accumulator_value)
-    mu = jax.tree_map(value_like, params)  # First moment
-    nu = jax.tree_map(value_like, params)  # Second Central moment
+    mu = jax.tree_util.tree_map(value_like, params)  # First moment
+    nu = jax.tree_util.tree_map(value_like, params)  # Second Central moment
     return ScaleByAdamState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
 
   def update_fn(updates, state, params=None):
     del params
-    mu = _update_moment(updates, state.mu, b1, 1)
-    nu = jax.tree_map(
+    mu = update_moment(updates, state.mu, b1, 1)
+    nu = jax.tree_util.tree_map(
         lambda g, v: v - (1 - b2) * jnp.sign(v - _abs_sq(g)) * _abs_sq(g),
         updates, state.nu)
     count_inc = numerics.safe_int32_increment(state.count)
-    mu_hat = _bias_correction(mu, b1, count_inc)
-    nu_hat = _bias_correction(nu, b2, count_inc)
-    updates = jax.tree_map(
+    mu_hat = bias_correction(mu, b1, count_inc)
+    nu_hat = bias_correction(nu, b2, count_inc)
+    updates = jax.tree_util.tree_map(
         lambda m, v: m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
     return updates, ScaleByAdamState(count=count_inc, mu=mu, nu=nu)
 
@@ -554,24 +600,24 @@ def scale_by_radam(
     mu_hat = params[1]
     nu_hat = params[2]
     r = jnp.sqrt((ro - 4)*(ro - 2)*ro_inf/((ro_inf - 4)*(ro_inf - 2)*ro))
-    updates = jax.tree_map(
+    updates = jax.tree_util.tree_map(
         lambda m, v: r*m / (jnp.sqrt(v + eps_root) + eps), mu_hat, nu_hat)
     return updates
 
   def init_fn(params):
-    mu = jax.tree_map(jnp.zeros_like, params)  # First moment
-    nu = jax.tree_map(jnp.zeros_like, params)  # Second moment
+    mu = jax.tree_util.tree_map(jnp.zeros_like, params)  # First moment
+    nu = jax.tree_util.tree_map(jnp.zeros_like, params)  # Second moment
     return ScaleByAdamState(count=jnp.zeros([], jnp.int32), mu=mu, nu=nu)
 
   def update_fn(updates, state, params=None):
     del params
-    mu = _update_moment(updates, state.mu, b1, 1)
-    nu = _update_moment_per_elem_norm(updates, state.nu, b2, 2)
+    mu = update_moment(updates, state.mu, b1, 1)
+    nu = update_moment_per_elem_norm(updates, state.nu, b2, 2)
     count_inc = numerics.safe_int32_increment(state.count)
     b2t = b2**count_inc
     ro = ro_inf - 2 * count_inc * b2t / (1 - b2t)
-    mu_hat = _bias_correction(mu, b1, count_inc)
-    nu_hat = _bias_correction(nu, b2, count_inc)
+    mu_hat = bias_correction(mu, b1, count_inc)
+    nu_hat = bias_correction(nu, b2, count_inc)
     updates = jax.lax.cond(
         ro >= threshold, _radam_update, lambda _: mu_hat,
         (ro, mu_hat, nu_hat))
@@ -607,7 +653,7 @@ def add_decayed_weights(
   def update_fn(updates, state, params):
     if params is None:
       raise ValueError(base.NO_PARAMS_MSG)
-    updates = jax.tree_map(
+    updates = jax.tree_util.tree_map(
         lambda g, p: g + weight_decay * p, updates, params)
     return updates, state
 
@@ -644,7 +690,7 @@ def scale_by_schedule(
   def update_fn(updates, state, params=None):
     del params
     step_size = step_size_fn(state.count)
-    updates = jax.tree_map(
+    updates = jax.tree_util.tree_map(
         lambda g: jnp.array(step_size, dtype=g.dtype) * g, updates)
     return updates, ScaleByScheduleState(
         count=numerics.safe_int32_increment(state.count))
@@ -703,7 +749,7 @@ def scale_by_trust_ratio(
 
       return update * safe_trust_ratio
 
-    updates = jax.tree_map(_scale_update, updates, params)
+    updates = jax.tree_util.tree_map(_scale_update, updates, params)
     return updates, state
 
   return base.GradientTransformation(init_fn, update_fn)
@@ -741,16 +787,16 @@ def add_noise(
 
   def update_fn(updates, state, params=None):  # pylint: disable=missing-docstring
     del params
-    num_vars = len(jax.tree_leaves(updates))
-    treedef = jax.tree_structure(updates)
+    num_vars = len(jax.tree_util.tree_leaves(updates))
+    treedef = jax.tree_util.tree_structure(updates)
     count_inc = numerics.safe_int32_increment(state.count)
     variance = eta / count_inc**gamma
     standard_deviation = jnp.sqrt(variance)
     all_keys = jax.random.split(state.rng_key, num=num_vars + 1)
-    noise = jax.tree_map(
+    noise = jax.tree_util.tree_map(
         lambda g, k: jax.random.normal(k, shape=g.shape, dtype=g.dtype),
-        updates, jax.tree_unflatten(treedef, all_keys[1:]))
-    updates = jax.tree_map(
+        updates, jax.tree_util.tree_unflatten(treedef, all_keys[1:]))
+    updates = jax.tree_util.tree_map(
         lambda g, n: g + standard_deviation.astype(g.dtype) * n,
         updates, noise)
     return updates, AddNoiseState(count=count_inc, rng_key=all_keys[0])
@@ -783,17 +829,17 @@ def apply_every(
   """
 
   def init_fn(params):
-    grad_acc = jax.tree_map(jnp.zeros_like, params)
+    grad_acc = jax.tree_util.tree_map(jnp.zeros_like, params)
     return ApplyEvery(count=jnp.zeros([], jnp.int32), grad_acc=grad_acc)
 
   def update_fn(updates, state, params=None):
     del params
     c = state.count % k
     acc = c != 0
-    grad_acc = jax.tree_map(
+    grad_acc = jax.tree_util.tree_map(
         lambda g, ga: acc * ga + g, updates, state.grad_acc)
     emit = c == (k - 1)
-    updates = jax.tree_map(lambda ga: emit * ga, grad_acc)
+    updates = jax.tree_util.tree_map(lambda ga: emit * ga, grad_acc)
     count_inc = numerics.safe_int32_increment(state.count)
     return updates, ApplyEvery(count=count_inc % k, grad_acc=grad_acc)
 
@@ -826,7 +872,7 @@ def centralize() -> base.GradientTransformation:
 
   def update_fn(updates, state, params=None):
     del params
-    updates = jax.tree_map(_subtract_mean, updates)
+    updates = jax.tree_util.tree_map(_subtract_mean, updates)
     return updates, state
 
   return base.GradientTransformation(init_fn, update_fn)
@@ -862,8 +908,8 @@ def scale_by_sm3(
 
   def init_fn(params):
     _reject_complex(params)
-    mu = jax.tree_map(zeros_for_dim, params)
-    nu = jax.tree_map(jnp.zeros_like, params)
+    mu = jax.tree_util.tree_map(zeros_for_dim, params)
+    nu = jax.tree_util.tree_map(jnp.zeros_like, params)
     return ScaleBySM3State(mu, nu)
 
   def _expanded_shape(shape, axis):
@@ -890,16 +936,16 @@ def scale_by_sm3(
 
   def update_fn(updates, state, params=None):
     del params
-    mu = jax.tree_map(
+    mu = jax.tree_util.tree_map(
         lambda g, v:  # pylint:disable=g-long-lambda
         [jnp.reshape(v[i], _expanded_shape(g.shape, i)) for i in range(g.ndim)],
         updates, state.mu)
-    accum = jax.tree_map(_new_accum, updates, mu)
-    accum_inv_sqrt = jax.tree_map(
+    accum = jax.tree_util.tree_map(_new_accum, updates, mu)
+    accum_inv_sqrt = jax.tree_util.tree_map(
         lambda t: jnp.where(t > 0, jax.lax.rsqrt(t + eps), 0.0), accum)
-    up = jax.tree_map(lambda g, a: g*a, updates, accum_inv_sqrt)
-    nu = _update_moment(up, state.nu, b1, 1)
-    mu = jax.tree_map(
+    up = jax.tree_util.tree_map(lambda g, a: g*a, updates, accum_inv_sqrt)
+    nu = update_moment(up, state.nu, b1, 1)
+    mu = jax.tree_util.tree_map(
         lambda g: [_new_mu(g, i) for i in range(g.ndim)], accum)
 
     return nu, ScaleBySM3State(mu=mu, nu=nu)
